/*
 * THIS FILE AUTOMATICALLY GENERATED. DO NOT EDIT.
 * generated from:
 *	OpenBSD: cpudevs,v 1.44 2007/05/18 15:57:35 kettenis Exp 
 */

{HPPA_TYPE_BOARD,	HPPA_BOARD_HP840,	"840/930 (Indigo)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP825,	"825/925 (FireFox)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP835,	"83[45]/[69]35 (TopGun)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP845,	"845/645 (Technical ShoGun)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP850,	"850/950 (Cheetah)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP949,	"949 (Commercial ShoGun)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP950S,	"950S (Cheetah)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP855,	"855/955 (Jaguar)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP860,	"860/960 (Cougar)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP870,	"865/870/980 (Panther)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP810,	"810 (Burgundy)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP822,	"822/922 (SilverFox Low)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP832,	"832/932 (SilverFox High)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP808,	"808/815/920 (Lego, SilverLite)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP842,	"842/948 (SilverBullet Low)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP852,	"852/958 (SilverBullet High)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HPOBOE,	"(Oboe)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HPDRAGON,	"(Dragon)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP890,	"890/990/992 (Chimera)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP891,	"891/T500 (TNT 100)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP892,	"892/T520 (TNT 120)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP893_T540,	"893/T540/T600 (Jade 180 U)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HPHXPROC,	"Hitachi X Processor" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP720,	"720 (Cobra)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP750_66,	"750/66 (Coral)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP730_66,	"730/66 (King Cobra)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP735_99,	"735/99 (Hardball)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP755_99,	"755/99 (Coral II)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP755_125,	"755/125 (Coral II)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP735_130,	"735/130 (Snake Cheetah)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP817,	"817/827/957/957LX/FH20 (Nova Low)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP837,	"837/847/857/967/967LX/FGHI30 (Nova High)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP807,	"807/917/917LX/927/927LX/937/937LX/947/947LX/F10 (Nova8)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP867,	"867/877/977/GHI40 (Nova64)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP887T,	"887/897/987/GHI50 (TNova)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP887,	"887/897/GHI70 (Hydra96 (Nova))" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP887T96,	"887/897/GHI60 (TNova96)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP710,	"710/50 (Bushmaster)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP705,	"705/35 (Flounder)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715_50,	"715/50 (Scorpio)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715_33,	"715/33 (Scorpio Jr.)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715S_50,	"715S/50 (Spider-50)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715S_33,	"715S/33 (Spider-33)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715T_50,	"715T/50 (Trailways-50)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715T_33,	"715T/33 (Trailways-33)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715_75,	"715/75 (Scorpio Sr.)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715_99,	"715/99 (Scorpio 100)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP725_50,	"725/50 (Spectra)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP725_75,	"725/75 (Spectra)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP725_99,	"725/99 (Spectra)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP745I_50,	"745i/747i/50 (Pace)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP742I_50,	"742i/50 (Sidewinder)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP747I_100,	"745i/747i/100 (Fast Pace)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HPE23,	"806/E23 (Orville)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HPE25,	"806/E25 (Wilbur)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HPE35,	"816/E35 (Wright Brothers 80)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HPE45,	"826/E45 (Wright Brothers 96)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HPD210,	"811/D210/D310 (UltraLight L 100)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HPD200,	"801/D200/D300 (UltraLight L 75)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP778_B132L,	"778/B132L (Merlin L2 132)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP778_B160L,	"778/B160L (Merlin L2 160)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP778_B132LP,	"778/B132L+ (Merlin L2+ 132)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP778_B180L,	"778/B180L+ (Merlin L2+ 180)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP779_C132L,	"779/C132L (Raven L2 132)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP779_C160L,	"779/C160L (Raven L2 160)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP779_C180L,	"779/C180L (Raven L2 180)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP779_C160L1,	"779/C160L (Raven L2 160)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP712_132L2,	"712/132 (L2 Upgrade)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP712_160L2,	"712/160 (L2 Upgrade)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP712_132L21,	"712/132 (L2 Upgrade)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP712_160L21,	"712/160 (L2 Upgrade)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP744_132L,	"744/132L (Anole-132L)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP744_165L,	"744/165L (Anole-165L)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP803,	"803/D220 (UltraLight L2 132)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP813,	"813/D320 (UltraLight L2 160)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP778_B132,	"778/B132 (Merlin Jr. L2 132)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_A132,	"800/A132 (Staccato L2 132)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_A180C,	"800/A180C (Staccato L2 180)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP809,	"809/K100 (KittyHawk DC2-100)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP839,	"839/K210 (KittyHawk DC3-120)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP829,	"829/K400 (KittyHawk DC3-100)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP849,	"849/K410 (KittyHawk DC3-120)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP770_J210XC,	"770/J210XC (LighteningHawk)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP770_J200,	"770/J200 (SkyHawk 100)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP770_J210,	"770/J210 (SkyHawk 120)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP841,	"841/D210/D410 (UltraLight Proc 1-way T'120)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP851,	"851/D210-2/D410-2 (UltraLight Proc 2-way T'120)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP821_1,	"821/D200/D400 (UltraLight Proc 1-way T'100)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP831,	"831/D200-2/D400-2 (UltraLight Proc 2-way T'100)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP819,	"819/K200 (KittyHawk DC2 100)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP859,	"859/K220 (ThunderHawk DC3- 120 1M)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP869,	"869/K420 (ThunderHawk DC3 120 1M)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP780_C110,	"780/C110 (Raven 120 T')" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP804,	"804/K450 (Mohawk 160 U 1M DC3)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP889,	"889/K460 (Mohawk 180 U 1M DC3)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP780_C100,	"780/C100 (Raven 100 T')" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP780_J282,	"780/J282 (FireHawk 180 U)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800V,	"800/V2200/V2500 (SPP2000)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP780_C230,	"780/C230 (Raven U 230)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP780_C240,	"780/C240 (Raven U 240)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP780_C160,	"780/C160 (Raven U 160)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP780_C180XP,	"780/C180XP (Raven U 180)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP780_C200,	"780/C200 (Raven U 200)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP780_C180P,	"780/C180+ (Raven U 180+)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP841_1,	"841/D260/D360 (UltraLight 1-way T120 1MB)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP851_1,	"851/D260/D360 (UltraLight 2-way T120 1MB)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP861,	"861/D270/D370 (UltraLight 1-way U160 512K)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP871,	"871/D270/D370 (UltraLight 2-way U160 512K)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP802,	"802/K250 (MoHawk U160 1M DC3-)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP879,	"879/K260 (MoHawk U180 1M DC3-)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP810D,	"810/D380/R380 (UltraLight 1-way U180 1M)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP820,	"820/D280 (UltraLight 2-way U180 1M)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP898,	"898/K370 (BraveHawk 200 DC3-)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP899,	"899/K570 (BraveHawk 200 DC3)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800V2,	"800/V2500 (SPP2500)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP782_J2240,	"782/J2240 (SummitHawk U+)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800K,	"800/K580 (DragonHawk U+ 240 DC3)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP821,	"821/D350/R350 (UltraLight 1w U+/240)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800D,	"800/D390/R390 (UltraLight 2w U+/240)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_C3000,	"785/C3000 (AllegroHigh W)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_B1000,	"785/B1000 (AllegroLow W)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_J5000,	"785/J5000 (Forte W 2w)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_N400036,	"800/N4000-36/rp7400 (Prelude W)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_J7000,	"785/J7000 (Forte W 4w)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_L200044,	"800/L2000-44/rp5450 (Rhapsody 440)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_L200036,	"800/L2000-36/rp5450 (Rhapsody 360)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_C360,	"780/C360 (Raven W 360)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_L100044,	"800/L1000-44/rp5400 (Rhapsody DC-440)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_L100036,	"800/L1000-36/rp5400 (Rhapsody DC-360)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP897_A50044,	"897/A500-44/rp2450 (Crescendo 440)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_N400044,	"800/N4000-44/rp7400 (Prelude W 440)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_C3600,	"785/C3600 (Allegro)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP786_B2000,	"786/B2000 (Kazoo W+)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_J5600,	"785/J5600 (Forte W+ 2w)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_J7600,	"785/J7600 (Forte W+ 4w)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_N400055,	"800/N4000-55/rp7400 (Prelude W+ 550)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_J6000,	"785/J6000 (Duet W+)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP897_A5005X,	"897/A500-5X/rp2450 (Crescendo 550)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP897_A40044,	"897/A400-44/rp2400 (Crescendo 440)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_N40005X,	"800/N4000-5X/rp7410 (Keystone W+ 550)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_L150044,	"800/L1500-44/rp5430 (Rhapsody wave 2 W+)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_L150036,	"800/L1500-36/rp5430 (Rhapsody wave 2 W+)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_L30005X,	"800/L3000-5X/rp5470 (Marcato W+)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_C3700,	"785/C3700" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_J6700,	"785/J6700 (Duet W2)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_B2600,	"785/B2600 (Piccolo W+)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP897_A4006X,	"897/A400-6X/rp2430 (Crescendo DC-W2)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP897_A5006X,	"897/A500-6X/rp2470 (Crescendo 650 W2)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP897_A5007X,	"897/A500-7X/rp2470 (Crescendo 750 W2)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_N40007X,	"800/N4000-7X/rp7410 (Keystone/Matterhorn W2 750)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP800_N40006X,	"800/N4000-6X/rp7410 (Keystone/Matterhorn W2 650)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HITACHIW,	"(Hitachi W)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP785_C8000,	"785/C8000" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP712_60,	"712/60 (Gecko)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP712_80,	"712/80 (King Gecko)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP712_100,	"712/100 (King Gecko)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP743I_64,	"743i/64 (Anole-64)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP743I_100,	"743i/100 (Anole-100)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP712_120,	"712/120 (King Gecko)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715_80,	"715/80 (Scorpio-L)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715_64,	"715/64 (Mirage Jr.)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715_100,	"715/100 (Mirage Sr.)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715_100XC,	"715/100XC (Turnip)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP725_100,	"725/100 (Electra)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP725_120,	"725/120 (Electra)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715_100L,	"715/100L (Scorpio-L)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715_120L,	"715/120L (Scorpio-L)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP725_80L,	"725/80L (Spectra-L)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP725_100L,	"725/100L (Spectra-L)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP725_120L,	"725/120L (Spectra-L)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP743_50,	"743/50 (Jason)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP743_100,	"743/100 (Jason)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP715_80M,	"715/80 (Mirage)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP811,	"811/DX5 (SAIC L-80)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP801,	"801/DX0 (Rocky1 L-80)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HP743T,	"743T (Anole T)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HITACHIT80,	"(Hitachi Tiny 80)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_HITACHIT64,	"(Hitachi Tiny 64)" },
{HPPA_TYPE_BOARD,	HPPA_BOARD_NEC,	"(NEC Aska)" },
{HPPA_TYPE_BUS,	HPPA_BUS_MID_BUS,	"MID_BUS" },
{HPPA_TYPE_BUS,	HPPA_BUS_SMB,	"SMB" },
{HPPA_TYPE_BUS,	HPPA_BUS_HPPB,	"HP-PB" },
{HPPA_TYPE_BUS,	HPPA_BUS_SUMMIT,	"Summit" },
{HPPA_TYPE_BUS,	HPPA_BUS_VVSC,	"Viper-VSC" },
{HPPA_TYPE_BUS,	HPPA_BUS_VHPPB,	"Viper-HPPB" },
{HPPA_TYPE_BUS,	HPPA_BUS_K9KSC,	"K9-KSC" },
{HPPA_TYPE_BUS,	HPPA_BUS_SPSSC,	"Spider-SSC" },
{HPPA_TYPE_BUS,	HPPA_BUS_STSSC,	"Stiletto-SSC" },
{HPPA_TYPE_BUS,	HPPA_BUS_GSCHPPB,	"GSC-HPPB" },
{HPPA_TYPE_BUS,	HPPA_BUS_GSCHSC,	"GSC+(HSC)" },
{HPPA_TYPE_BUS,	HPPA_BUS_RUNWAY,	"Runway" },
{HPPA_TYPE_BUS,	HPPA_BUS_GSC,	"GSC" },
{HPPA_TYPE_BUS,	HPPA_BUS_GSCP,	"GSC++" },
{HPPA_TYPE_BUS,	HPPA_BUS_NECSB,	"NEC System Bus" },
{HPPA_TYPE_BUS,	HPPA_BUS_ROPES,	"Ropes" },
{HPPA_TYPE_BUS,	HPPA_BUS_MERCED,	"Merced" },
{HPPA_TYPE_CPU,	HPPA_CPU_SOFTEMU,	"Software Emulation" },
{HPPA_TYPE_CPU,	HPPA_CPU_PCXT2,	"PCXT' (Thunderbird')" },
{HPPA_TYPE_CPU,	HPPA_CPU_PCXL,	"PCXL (Hummingbird)" },
{HPPA_TYPE_CPU,	HPPA_CPU_PCXU,	"PCXU (Onyx)" },
{HPPA_TYPE_CPU,	HPPA_CPU_PCXL2,	"PCXL2 (Velociraptor)" },
{HPPA_TYPE_CPU,	HPPA_CPU_PCXUP,	"PCXU+ (Vulcan)" },
{HPPA_TYPE_CPU,	HPPA_CPU_PCXW,	"PCXW (Vulcan)" },
{HPPA_TYPE_CPU,	HPPA_CPU_PCXW2,	"PCXW2 (Piranha)" },
{HPPA_TYPE_FPU,	HPPA_FPU_SOFTEMU,	"Software Emulation" },
{HPPA_TYPE_FPU,	HPPA_FPU_INDIGO,	"Indigo" },
{HPPA_TYPE_FPU,	HPPA_FPU_TYCO,	"Sterling I MIU (TYCO)" },
{HPPA_TYPE_FPU,	HPPA_FPU_ROC,	"Sterling I MIU (ROC w/Weitek)" },
{HPPA_TYPE_FPU,	HPPA_FPU_FPCW,	"FPC (NMOS-III w/Weitek)" },
{HPPA_TYPE_FPU,	HPPA_FPU_FPCB,	"FPC (NMOS-III w/Bit)" },
{HPPA_TYPE_FPU,	HPPA_FPU_PCXS,	"PCXS (Timex-II)" },
{HPPA_TYPE_FPU,	HPPA_FPU_PCXT,	"PCXT (Rolex - CMOS-26B)" },
{HPPA_TYPE_FPU,	HPPA_FPU_HARP,	"HARP-I" },
{HPPA_TYPE_FPU,	HPPA_FPU_PCXT2,	"PCXT' (Tornado)" },
{HPPA_TYPE_FPU,	HPPA_FPU_PA50,	"PA-50 (Hitachi)" },
{HPPA_TYPE_FPU,	HPPA_FPU_PCXL,	"PCXL (CMOS-26B)" },
{HPPA_TYPE_FPU,	HPPA_FPU_PCXU,	"PCXU/PCXU+" },
{HPPA_TYPE_FPU,	HPPA_FPU_VR,	"VR" },
{HPPA_TYPE_FPU,	HPPA_FPU_PCXW,	"PCXW" },
{HPPA_TYPE_FPU,	HPPA_FPU_PCXW2,	"PCXW2" },
{HPPA_TYPE_NPROC,	HPPA_NPROC_HPPA,	"PA-RISC" },
{HPPA_TYPE_MEMORY,	HPPA_MEMORY_ARCH,	"architected memory module" },
{HPPA_TYPE_MEMORY,	HPPA_MEMORY_PDEP,	"pdep memory module" },
{HPPA_TYPE_BDMA,	HPPA_BDMA_PARALLEL,	"Parallel I/O" },
{HPPA_TYPE_BDMA,	HPPA_BDMA_PRDB,	"Parallel RDB" },
{HPPA_TYPE_BDMA,	HPPA_BDMA_MIDBUSPSI,	"MID_BUS PSI" },
{HPPA_TYPE_BDMA,	HPPA_BDMA_HPPBTPSI,	"HP-PB Transit PSI" },
{HPPA_TYPE_BDMA,	HPPA_BDMA_HPPBT8023,	"HP-PB Transit 802.3" },
{HPPA_TYPE_BDMA,	HPPA_BDMA_HPPBLANCON,	"Miura LAN/Console (J2146A)" },
{HPPA_TYPE_BDMA,	HPPA_BDMA_HPPB8024,	"HP-PB Transit 802.4" },
{HPPA_TYPE_BDMA,	HPPA_BDMA_LANCONS,	"LAN/Console" },
{HPPA_TYPE_BDMA,	HPPA_BDMA_EXERCISER,	"GSC+ Exerciser" },
{HPPA_TYPE_BDMA,	HPPA_BDMA_HPPBHFI,	"HP-PB HF Interface" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_SCSI,	"SCSI" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_FWSCSI0,	"Wizard FW SCSI" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_HPIB,	"HP-IB" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_HPPBFL,	"HP-PB HP-FL" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_HPPBIB,	"HP-PB LoQuix HP-IB" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_HPPBCL,	"HP-PB Crypt LoQuix" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_HPPBGPS,	"HP-PB Shazam GPIO" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_HPPBGPL,	"HP-PB LoQuix GPIO" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_2X25,	"2-Port X.25 NIO_ACC" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_4X25,	"4-Port X.25 NIO_ACC" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_NIMBFB,	"Nimbus Frame Buffer" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_NIMBCL,	"Nimbus Control" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_MRTI,	"Martian RTI" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_ACCMUX,	"ACC Mux" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_L8023,	"Lanbrusca 802.3" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_LQFDDI,	"HP-PB LoQuix FDDI" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_LQFDDI1,	"HP-PB LoQuix FDDI (28670A)" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_GTR,	"TI TokenRing" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_FWSCSI,	"FW SCSI" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_NETFC,	"Net FC" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_YATM,	"GSC+ YLIASTER ATM" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_MYRI,	"Hamlyn GSC+ Network Card" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_LFEMU,	"Lo-fat Emulator" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_VATM,	"GSC+ Venus ATM" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_HSCPCI,	"HP HSC-PCI Cards" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_ATMGSC,	"Alaxis GSC+ ATM" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_SHUSB,	"SummitHawk 230 USB" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_SHFETH,	"SummitHawk 230 100BaseT" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_MASSFC,	"Add-on mass FC" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_IPFC,	"Interphase NIO-FC" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_IPATM,	"Interphase NIO-ATM" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_IP100T,	"Interphase NIO-100BaseT" },
{HPPA_TYPE_ADMA,	HPPA_ADMA_IPNIOVG,	"Interphase NIO-100VG" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_APMUX,	"AP/MUX" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_RS232,	"RS-232" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_PEACOCK,	"Peacock Graphics" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_HIL,	"HIL" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_LEONARDO,	"Leonardo" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_HRM,	"HP-PB HRM" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_HRC,	"HP-PB HRC" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_LPT,	"Centronics interface" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_ATTDK,	"AT&T DataKit" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_HPIB,	"Test&Meas GSC HPIB" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_ROCKYKBD,	"Rocky2 Front Keyboard" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_HICONS,	"Hitachi Console Module" },
{HPPA_TYPE_ADIRECT,	HPPA_ADIRECT_HIBOOT,	"Hitachi Boot Module" },
{HPPA_TYPE_OTHER,	HPPA_OTHER_MIDBVM,	"MID_BUS Verification Master" },
{HPPA_TYPE_OTHER,	HPPA_OTHER_MIDBVS,	"MID_BUS Verification Slave" },
{HPPA_TYPE_OTHER,	HPPA_OTHER_MIDBEDU,	"MID_BUS Verification EDU" },
{HPPA_TYPE_OTHER,	HPPA_OTHER_SPECFB,	"Spectograph frame buffer" },
{HPPA_TYPE_OTHER,	HPPA_OTHER_SPECCTL,	"Spectograph control" },
{HPPA_TYPE_BCPORT,	HPPA_BCPORT_PHANTOM,	"Phantom Port" },
{HPPA_TYPE_BCPORT,	HPPA_BCPORT_PORT,	"Bus Converter Port" },
{HPPA_TYPE_CIO,	HPPA_CIO_CIO,	"CIO" },
{HPPA_TYPE_CONSOLE,	HPPA_CONSOLE_CHEETAH,	"Cheetah Console" },
{HPPA_TYPE_CONSOLE,	HPPA_CONSOLE_NULL,	"Jason/Anole Null Console" },
{HPPA_TYPE_FIO,	HPPA_FIO_A_X25,	"Armyknife Optional X.25" },
{HPPA_TYPE_FIO,	HPPA_FIO_8X25,	"8-Port X.25 EISA-ACC" },
{HPPA_TYPE_FIO,	HPPA_FIO_SCSI,	"NCR 53C700 SCSI" },
{HPPA_TYPE_FIO,	HPPA_FIO_LAN,	"Core LAN" },
{HPPA_TYPE_FIO,	HPPA_FIO_HIL,	"Core HIL" },
{HPPA_TYPE_FIO,	HPPA_FIO_CENT,	"Core Centronics" },
{HPPA_TYPE_FIO,	HPPA_FIO_RS232,	"Core RS-232C" },
{HPPA_TYPE_FIO,	HPPA_FIO_SGC,	"SGC Graphics" },
{HPPA_TYPE_FIO,	HPPA_FIO_A1,	"Core audio (int.)" },
{HPPA_TYPE_FIO,	HPPA_FIO_A2NB,	"Advanced audio (ext.)" },
{HPPA_TYPE_FIO,	HPPA_FIO_FWSCSI,	"NCR 53C720 FW-SCSI" },
{HPPA_TYPE_FIO,	HPPA_FIO_FDDI,	"Core FDDI" },
{HPPA_TYPE_FIO,	HPPA_FIO_A1NB,	"Core audio (ext.)" },
{HPPA_TYPE_FIO,	HPPA_FIO_A2,	"Advanced audio (int.)" },
{HPPA_TYPE_FIO,	HPPA_FIO_HPIB,	"Core HPIB" },
{HPPA_TYPE_FIO,	HPPA_FIO_TR9000,	"GSC IBM TokenRing" },
{HPPA_TYPE_FIO,	HPPA_FIO_VMENET,	"VME Networking" },
{HPPA_TYPE_FIO,	HPPA_FIO_BOOTROM,	"BOOT ROM" },
{HPPA_TYPE_FIO,	HPPA_FIO_DINOPCK,	"Dino PC Keyboard" },
{HPPA_TYPE_FIO,	HPPA_FIO_EISALAN,	"Cascade EISA 100VG lan" },
{HPPA_TYPE_FIO,	HPPA_FIO_WHPIB,	"Wax/Clark/Dino HPIB" },
{HPPA_TYPE_FIO,	HPPA_FIO_SPPCON,	"SPP2000 Console TTY" },
{HPPA_TYPE_FIO,	HPPA_FIO_PCILAN,	"PCI Plug-In LAN" },
{HPPA_TYPE_FIO,	HPPA_FIO_PCIDSK,	"PCI Plug-In Disk" },
{HPPA_TYPE_FIO,	HPPA_FIO_SPPCORE,	"SPP2000 Core BA" },
{HPPA_TYPE_FIO,	HPPA_FIO_SONICET,	"Sonic Ethernet" },
{HPPA_TYPE_FIO,	HPPA_FIO_PCIMISC,	"Misc PCI Plug-in" },
{HPPA_TYPE_FIO,	HPPA_FIO_METHEUS,	"Metheus Frame Buffer" },
{HPPA_TYPE_FIO,	HPPA_FIO_BARCO,	"BARCO CX4500 VME Graphics Console" },
{HPPA_TYPE_FIO,	HPPA_FIO_HUGHES,	"Hughes TOG VME FDDI" },
{HPPA_TYPE_FIO,	HPPA_FIO_GSCSI,	"NCR 53C710 SCSI" },
{HPPA_TYPE_FIO,	HPPA_FIO_GPCFD,	"floppy controller" },
{HPPA_TYPE_FIO,	HPPA_FIO_GPCIO,	"PS/2 port" },
{HPPA_TYPE_FIO,	HPPA_FIO_GSGC,	"GIO Graphics" },
{HPPA_TYPE_FIO,	HPPA_FIO_GISDN,	"Core ISDN" },
{HPPA_TYPE_FIO,	HPPA_FIO_GSCSCSI,	"GSC SCSI" },
{HPPA_TYPE_FIO,	HPPA_FIO_GLAN,	"Core LAN" },
{HPPA_TYPE_FIO,	HPPA_FIO_GRS232,	"Core RS-232C" },
{HPPA_TYPE_FIO,	HPPA_FIO_GRJ16,	"Optional RJ-16" },
{HPPA_TYPE_BHA,	HPPA_BHA_DIO,	"DIO Bus Adapter" },
{HPPA_TYPE_BHA,	HPPA_BHA_SGC,	"SGC Bus Adapter" },
{HPPA_TYPE_BHA,	HPPA_BHA_GSC,	"GSC Bus Adapter" },
{HPPA_TYPE_BHA,	HPPA_BHA_ASP,	"Core Bus Adapter (ASP)" },
{HPPA_TYPE_BHA,	HPPA_BHA_EISA,	"EISA Bus Adapter" },
{HPPA_TYPE_BHA,	HPPA_BHA_VME,	"VME Bus Adapter" },
{HPPA_TYPE_BHA,	HPPA_BHA_LASI,	"LASI Bus Adapter" },
{HPPA_TYPE_BHA,	HPPA_BHA_PCMCIA,	"PCMCIA Bus Adapter" },
{HPPA_TYPE_BHA,	HPPA_BHA_WAX,	"WAX Bus Adapter" },
{HPPA_TYPE_BHA,	HPPA_BHA_WEISA,	"WAX EISA Bus Adapter" },
{HPPA_TYPE_BHA,	HPPA_BHA_TIMI,	"TIMI Bus Adapter" },
{HPPA_TYPE_BHA,	HPPA_BHA_EPIC,	"Epic PCI Bus Adapter" },
{HPPA_TYPE_IOA,	HPPA_IOA_UTURN,	"U2/UTurn Runway BC Port" },
{HPPA_TYPE_IOA,	HPPA_IOA_PLUTO,	"Pluto BC Port" },
{HPPA_TYPE_BRIDGE,	HPPA_BRIDGE_DINO,	"PCI Bridge" },
{HPPA_TYPE_BRIDGE,	HPPA_BRIDGE_SAGA,	"Saga PCI Bridge" },
{HPPA_TYPE_FABRIC,	HPPA_FABRIC_DNACA,	"Halfdome DNA Central Agent" },
{HPPA_TYPE_FABRIC,	HPPA_FABRIC_TOGO,	"Halfdome TOGO Fabric Crossbar" },
{HPPA_TYPE_FABRIC,	HPPA_FABRIC_SAKURA,	"Halfdome Sakura Fabric Router" },
{ -1 }
